@@ -6,14 +6,21 @@ import (
 	"errors"
 	"fmt"
 	"os"
+	"regexp"
 	"strings"
 	"sync"
 	"sync/atomic"
 	"time"
 
+	"github.com/nozzle/throttler"
+	"github.com/urfave/cli/v2"
+	"golang.org/x/exp/slices"
+	"golang.org/x/net/idna"
+
 	"github.com/StackExchange/dnscontrol/v4/models"
 	"github.com/StackExchange/dnscontrol/v4/pkg/bindserial"
 	"github.com/StackExchange/dnscontrol/v4/pkg/credsfile"
+	"github.com/StackExchange/dnscontrol/v4/pkg/domaintags"
 	"github.com/StackExchange/dnscontrol/v4/pkg/nameservers"
 	"github.com/StackExchange/dnscontrol/v4/pkg/normalize"
 	"github.com/StackExchange/dnscontrol/v4/pkg/notifications"
@@ -21,10 +28,6 @@ import (
 	"github.com/StackExchange/dnscontrol/v4/pkg/rfc4183"
 	"github.com/StackExchange/dnscontrol/v4/pkg/zonerecs"
 	"github.com/StackExchange/dnscontrol/v4/providers"
-	"github.com/nozzle/throttler"
-	"github.com/urfave/cli/v2"
-	"golang.org/x/exp/slices"
-	"golang.org/x/net/idna"
 )
 
 type cmdZoneCache struct {
@@ -54,7 +57,6 @@ type PPreviewArgs struct {
 	ConcurMode        string
 	ConcurMax         int // Maximum number of concurrent connections
 	NoPopulate        bool
-	DePopulate        bool
 	PopulateOnPreview bool
 	Report            string
 	Full              bool
@@ -113,11 +115,6 @@ func (args *PPreviewArgs) flags() []cli.Flag {
 		Destination: &args.NoPopulate,
 		Usage:       `Do not auto-create zones at the provider`,
 	})
-	flags = append(flags, &cli.BoolFlag{
-		Name:        "depopulate",
-		Destination: &args.NoPopulate,
-		Usage:       `Delete unknown zones at provider (dangerous!)`,
-	})
 	flags = append(flags, &cli.BoolFlag{
 		Name:        "populate-on-preview",
 		Destination: &args.PopulateOnPreview,
@@ -292,7 +289,7 @@ func prun(args PPreviewArgs, push bool, interactive bool, out printer.CLI, repor
 					continue // Do not emit noise when zone exists
 				}
 				if !started {
-					out.StartDomain(zone.GetUniqueName())
+					out.StartDomain(zone)
 					started = true
 				}
 				skip := skipProvider(provider.Name, providersToProcess)
@@ -300,7 +297,7 @@ func prun(args PPreviewArgs, push bool, interactive bool, out printer.CLI, repor
 				if !skip {
 					totalCorrections += len(corrections)
 					out.EndProvider2(provider.Name, len(corrections))
-					reportItems = append(reportItems, genReportItem(zone.Name, corrections, provider.Name))
+					reportItems = append(reportItems, genReportItem(zone.Name, corrections, provider.Name, ""))
 					anyErrors = cmp.Or(anyErrors, pprintOrRunCorrections(zone.Name, provider.Name, corrections, out, push || args.PopulateOnPreview, interactive, notifier, report))
 				}
 			}
@@ -355,7 +352,7 @@ func prun(args PPreviewArgs, push bool, interactive bool, out printer.CLI, repor
 	// Now we know what to do, print or do the tasks.
 	out.PrintfIf(fullMode, "PHASE 3: CORRECTIONS\n")
 	for _, zone := range zonesToProcess {
-		out.StartDomain(zone.GetUniqueName())
+		out.StartDomain(zone)
 
 		// Process DNS provider changes:
 		providersToProcess := whichProvidersToProcess(zone.DNSProviderInstances, args.Providers)
@@ -367,7 +364,7 @@ func prun(args PPreviewArgs, push bool, interactive bool, out printer.CLI, repor
 				numActions := zone.GetChangeCount(provider.Name)
 				totalCorrections += numActions
 				out.EndProvider2(provider.Name, numActions)
-				reportItems = append(reportItems, genReportItem(zone.Name, corrections, provider.Name))
+				reportItems = append(reportItems, genReportItem(zone.Name, corrections, provider.Name, ""))
 				anyErrors = cmp.Or(anyErrors, pprintOrRunCorrections(zone.Name, provider.Name, corrections, out, push, interactive, notifier, report))
 			}
 		}
@@ -380,7 +377,7 @@ func prun(args PPreviewArgs, push bool, interactive bool, out printer.CLI, repor
 			numActions := zone.GetChangeCount(zone.RegistrarInstance.Name)
 			out.EndProvider2(zone.RegistrarName, numActions)
 			totalCorrections += numActions
-			reportItems = append(reportItems, genReportItem(zone.Name, corrections, zone.RegistrarName))
+			reportItems = append(reportItems, genReportItem(zone.Name, corrections, "", zone.RegistrarName))
 			anyErrors = cmp.Or(anyErrors, pprintOrRunCorrections(zone.Name, zone.RegistrarInstance.Name, corrections, out, push, interactive, notifier, report))
 		}
 	}
@@ -404,29 +401,16 @@ func prun(args PPreviewArgs, push bool, interactive bool, out printer.CLI, repor
 	return nil
 }
 
-// func countActions(corrections []*models.Correction) int {
-//	r := 0
-//	for _, c := range corrections {
-//		if c.F != nil {
-//			r++
-//		}
-//	}
-//	return r
-//}
-
 // whichZonesToProcess takes a list of DomainConfigs and a filter string and
-// returns a list of DomainConfigs whose metadata[DomainUniqueName] matched the
+// returns a list of DomainConfigs whose Domain.UniqueName matched the
 // filter. The filter string is a comma-separated list of domain names. If the
 // filter string is empty or "all", all domains are returned.
 func whichZonesToProcess(domains []*models.DomainConfig, filter string) []*models.DomainConfig {
-	if filter == "" || filter == "all" {
-		return domains
-	}
+	fh := domaintags.CompilePermitList(filter)
 
-	permitList := strings.Split(filter, ",")
 	var picked []*models.DomainConfig
 	for _, domain := range domains {
-		if domainInList(domain.GetUniqueName(), permitList) {
+		if fh.Permitted(domain.GetUniqueName()) {
 			picked = append(picked, domain)
 		}
 	}
@@ -564,19 +548,51 @@ func skipProvider(name string, providers []*models.DNSProviderInstance) bool {
 	})
 }
 
-func genReportItem(zname string, corrections []*models.Correction, pname string) *ReportItem {
-	// Only count the actions, not the messages.
-	cnt := 0
+func parseCorrectionMsg(s string) []string {
+	// Regex to remove the terminal styled formatting
+	ansiRe := regexp.MustCompile(`\x1b\[[0-9;]*[a-zA-Z]`)
+	s = ansiRe.ReplaceAllString(s, "")
+	// Create a slice(array) of correction/actions/changes from Msg
+	corrections := strings.Split(s, "\n")
+	// Clean up the slice, precaution remove any empty entries.
+	clean := make([]string, 0, len(corrections))
+	for _, l := range corrections {
+		l = strings.TrimSpace(l)
+		if l != "" {
+			clean = append(clean, l)
+		}
+	}
+	return clean
+}
+
+func genReportItem(zoneName string, corrections []*models.Correction, providerName string, registrarName string) *ReportItem {
+	correctionDetails := make([]string, 0)
 	for _, cor := range corrections {
 		if cor.F != nil {
-			cnt++
+			// `corrections` is a list that contains "informational" messages
+			// (where `.F = nil`) and "actions" to be taken (where `.F != nil`).
+			// When `.F = nil`, the contents of `.Msg` can either be a concatenation of all
+			// actions(all changes done in a single API call) or a single
+			// action(one API call per change), depending on the provider's implementation.
+			//
+			// We are parsing `cor.Msg` to remove terminal styled formatting and create
+			// a comprehensive list of actions (changes), as well as get an accurate
+			// number of corrections (`len(correctionDetails)`).
+
+			correctionDetails = append(correctionDetails, parseCorrectionMsg(cor.Msg)...)
 		}
 	}
 
 	r := ReportItem{
-		Domain:      zname,
-		Corrections: cnt,
-		Provider:    pname,
+		Domain:            zoneName,
+		Corrections:       len(correctionDetails),
+		CorrectionDetails: correctionDetails,
+	}
+	if providerName != "" {
+		r.Provider = providerName
+	}
+	if registrarName != "" {
+		r.Registrar = registrarName
 	}
 	return &r
 }
@@ -635,11 +651,13 @@ func writeReport(report string, reportItems []*ReportItem) error {
 		return err
 	}
 	defer f.Close()
-	b, err := json.MarshalIndent(reportItems, "", "  ")
-	if err != nil {
-		return err
-	}
-	if _, err := f.Write(b); err != nil {
+
+	// Disabling HTML encoding
+	enc := json.NewEncoder(f)
+	enc.SetIndent("", "  ")
+	enc.SetEscapeHTML(false)
+
+	if err := enc.Encode(reportItems); err != nil {
 		return err
 	}
 	return nil
